# smacof analysis of 1988 wine data
# Lew Harvey
# 3 February 2016

library("smacof")
library("rgl")

# rm(list = ls())

fn <- "wine_88.csv"
df <- read.csv(fn, header = TRUE)
df$w10 <- as.numeric(df$w10)

# read in wine properties
df.prop <- read.csv("wine_properties.csv", header = TRUE)
df.prop$wine <- as.character(df.prop$wine)
df.prop$color <- as.character(df.prop$color)

# loop through the data frame to
# create separate matrices for the
# data sets of each subject

nstim <- ncol(df)
nsets <- nrow(df) / ncol(df)
mlist <- list(NA)
mm <- list(NA)
winename <- colnames(df)

for (i in 1:nsets){
    mm[i] <- paste("m", i, sep = "")
    i1 <- ((i - 1) * nstim) + 1
    i2 <- i1 + nstim - 1
    mlist[i] <- list(mm = as.matrix(df[i1:i2,]))
    row.names(mlist[[i]]) <- winename
    mlist[[i]] <- as.dist(mlist[[i]])
}
names(mlist) <- unlist(mm)


# 2D solution
# -------------------------------------------------------
wine2D <- smacofIndDiff(mlist,
                      ndim = 2,
                      type = "ordinal",
                      constraint = "indscal",
                      itmax = 1000)

par(mfcol = c(2, 2))
par(pty = "s")
plot(wine2D, type = "p",
     plot.type = "confplot",
     label.conf = c(TRUE, 1, 1),
     xlim = c(-0.9, 0.9),
     ylim = c(-0.9, 0.9))
abline(h = 0, col = "gray")
abline(v = 0, col = "gray")
par(pty = "m")

plot(wine2D, plot.type = "Shepard")	
plot(wine2D, plot.type = "stressplot", ylim = c(0, 25))
plot(wine2D, plot.type = "resplot")
par(mfcol = c(1, 1))

# configuration plot without color
par(pty = "s")
plot(wine2D, type = "p", pch = 19,
     plot.type = "confplot",
     label.conf = c(TRUE, 1, 1),
     xlim = c(-0.9, 0.9),
     ylim = c(-0.9, 0.9))
abline(h = 0, col = "gray")
abline(v = 0, col = "gray")
par(pty = "m")

# configuration plot with color
par(pty = "s")
plot(wine2D, type = "p", pch = 19,
     col = df.prop$color,
     plot.type = "confplot",
     label.conf = c(TRUE, 1, 1),
     xlim = c(-0.9, 0.9),
     ylim = c(-0.9, 0.9))
abline(h = 0, col = "gray")
abline(v = 0, col = "gray")
par(pty = "m")

# 3D solution
# ----------------------------------------
wine3D <- smacofIndDiff(mlist,
                        ndim = 3,
                        type = "ordinal",
                        constraint = "indscal",
                        itmax = 5000)


plot3d(wine3D$gspace, type = "s",
       expand = 1.3,
       col = df.prop$color,
       xlab = "X",
       ylab = "Y",
       zlab = "Z")
text3d(wine3D$gspace,
       text = df.prop$wine,
       adj = c(1.5, 1.5))
